@@ -26,6 +26,7 @@ from amd_debug.common import (
     apply_prefix_wrapper,
     BIT,
     clear_temporary_message,
+    find_ip_version,
     get_distro,
     get_pretty_distro,
     get_property_pyudev,
@@ -48,6 +49,7 @@ from amd_debug.failures import (
     DevSlpDiskIssue,
     DevSlpHostIssue,
     DMArNotEnabled,
+    DmcubTooOld,
     DmiNotSetup,
     FadtWrong,
     I2CHidBug,
@@ -134,20 +136,24 @@ class PrerequisiteValidator(AmdTool):
             return True
 
         for name, p in edids.items():
-            try:
-                cmd = ["edid-decode", p]
-                output = subprocess.check_output(cmd, stderr=subprocess.DEVNULL).decode(
-                    "utf-8"
-                )
-            except FileNotFoundError:
-                self.db.record_prereq(
-                    "edid-decode not installed, unable to decode EDID", "👀"
-                )
-                return True
-            except subprocess.CalledProcessError as e:
-                self.db.record_prereq(f"Failed to capture EDID table: {e.output}", "👀")
-                return False
-            self.db.record_debug(apply_prefix_wrapper(f"EDID for {name}:", output))
+            output = None
+            for cmd in ["di-edid-decode", "edid-decode"]:
+                try:
+                    cmd = ["edid-decode", p]
+                    output = subprocess.check_output(
+                        cmd, stderr=subprocess.DEVNULL
+                    ).decode("utf-8")
+                    break
+                except FileNotFoundError:
+                    self.db.record_debug(f"{cmd} not installed")
+                except subprocess.CalledProcessError as e:
+                    self.db.record_debug(
+                        f"failed to capture edid with {cmd}: {e.output}"
+                    )
+            if not output:
+                self.db.record_prereq("Failed to capture EDID table", "👀")
+            else:
+                self.db.record_debug(apply_prefix_wrapper(f"EDID for {name}:", output))
         return True
 
     def check_amdgpu(self):
@@ -395,6 +401,53 @@ class PrerequisiteValidator(AmdTool):
             )
         return True
 
+    def check_dpia_pg_dmcub(self):
+        """Check if DMUB is new enough to PG DPIA when no USB4 present"""
+        usb4_found = False
+        for device in self.pyudev.list_devices(subsystem="pci", PCI_CLASS="C0340"):
+            usb4_found = True
+            break
+        if usb4_found:
+            self.db.record_debug("USB4 routers found, no need to check DMCUB version")
+            return True
+        # Check if matching DCN present
+        for device in self.pyudev.list_devices(subsystem="pci"):
+            current = None
+            klass = device.properties.get("PCI_CLASS")
+            if klass not in ["30000", "38000"]:
+                continue
+            pci_id = device.properties.get("PCI_ID")
+            if not pci_id.startswith("1002"):
+                continue
+            hw_ver = {"major": 3, "minor": 5, "revision": 0}
+            if not find_ip_version(device.sys_path, "DMU", hw_ver):
+                continue
+
+            # DCN was found, lookup version from sysfs
+            p = os.path.join(device.sys_path, "fw_version", "dmcub_fw_version")
+            if os.path.exists(p):
+                current = int(read_file(p), 16)
+
+            # no sysfs, try to look for version from debugfs
+            if not current:
+                slot = device.properties["PCI_SLOT_NAME"]
+                p = os.path.join(
+                    "/", "sys", "kernel", "debug", "dri", slot, "amdgpu_firmware_info"
+                )
+                contents = read_file(p)
+                for line in contents.split("\n"):
+                    if not line.startswith("DMCUB"):
+                        continue
+                    current = int(line.split()[-1], 16)
+            if current:
+                expected = 0x09001B00
+                if current >= expected:
+                    return True
+                self.db.record_prereq("DMCUB Firmware is outdated", "❌")
+                self.failures += [DmcubTooOld(current, expected)]
+                return False
+        return True
+
     def check_usb4(self):
         """Check if the thunderbolt driver is loaded"""
         slots = []
@@ -698,9 +751,13 @@ class PrerequisiteValidator(AmdTool):
             interface = device.properties.get("INTERFACE")
             cmd = ["ethtool", interface]
             wol_supported = False
-            output = subprocess.check_output(cmd, stderr=subprocess.DEVNULL).decode(
-                "utf-8"
-            )
+            try:
+                output = subprocess.check_output(cmd, stderr=subprocess.DEVNULL).decode(
+                    "utf-8"
+                )
+            except FileNotFoundError:
+                self.db.record_prereq(f"ethtool is missing", "👀")
+                return True
             for line in output.split("\n"):
                 if "Supports Wake-on" in line:
                     val = line.split(":")[1].strip()
@@ -949,6 +1006,8 @@ class PrerequisiteValidator(AmdTool):
                         stderr=subprocess.DEVNULL,
                     )
                     self.db.record_debug_file(f"{prefix}.dsl")
+                except FileNotFoundError as e:
+                    self.db.record_prereq(f"Failed to capture ACPI table: {e}", "👀")
                 except subprocess.CalledProcessError as e:
                     self.db.record_prereq(
                         f"Failed to capture ACPI table: {e.output}", "👀"
@@ -1099,13 +1158,16 @@ class PrerequisiteValidator(AmdTool):
                 self.db.record_prereq("IOMMU disabled", "✅")
                 return True
             debug_str += "DMA protection:\n"
-            for dev in self.pyudev.list_devices(
-                subsystem="thunderbolt", DEVTYPE="thunderbolt_domain"
-            ):
-                p = os.path.join(dev.sys_path, "iommu_dma_protection")
-                v = int(read_file(p))
-                debug_str += f"\t{p}: {v}\n"
-                found_dmar = v == 1
+            p = os.path.join("/", "sys", "firmware", "acpi", "tables", "IVRS")
+            with open(p, "rb") as f:
+                data = f.read()
+            if len(data) < 40:
+                raise ValueError(
+                    "IVRS table appears too small to contain virtualization info."
+                )
+            virt_info = struct.unpack_from("I", data, 36)[0]
+            debug_str += f"Virtualization info: 0x{virt_info:x}"
+            found_dmar = (virt_info & 0x2) != 0
             self.db.record_debug(debug_str)
             if not found_dmar:
                 self.db.record_prereq(
@@ -1205,6 +1267,7 @@ class PrerequisiteValidator(AmdTool):
                 self.check_smt,
                 self.check_iommu,
                 self.check_asus_rog_ally,
+                self.check_dpia_pg_dmcub,
             ]
 
         checks += [
